import API from '@/api'
import Web3 from 'web3'
import { showError } from '@/util/util'

let ethHelper = {
  isInitEth: false,
  curAccount: '',
  /**
   * @des 签名认证
   */
  async initEth () {
    if (!window.ethereum || !window.web3) {
      return Promise.reject(new Error('你还没有安装eth插件'))
    }

    if (!this.isInitEth) {
      if (window.ethereum) {
        window.web3 = new Web3(window.ethereum)
        try {
          await window.ethereum.enable()
        } catch (error) {
        }
      } else if (window.web3) {
        window.web3 = new Web3(window.web3.currentProvider)
      }

      this.isInitEth = true
    }

    let accounts = await window.web3.eth.getAccounts()
    let account = this.account = accounts[0]

    if (account) {
      this.curAccount = account
      let randomRes = await API.user.getRandom2({ account: account })
      let str = window.web3.utils.sha3(randomRes.data.data.random)

      return new Promise((resolve, reject) => {
        try {
          window.web3.eth.sign(str, account, (err, rs) => {
            if (err) {
              console.log('err1', err)
              reject(err)
            } else {
              resolve({
                account: account,
                sign: rs
              })
            }
          })
        } catch (err) {
          console.log('err2', err)
          reject(err)
        }
      })
    } else {
      return Promise.reject(new Error(''))
    }
  },
  /**
   * @des 转账
   * @param {Number} money 转账金额
   */
  async doTransfer (money) {
    try {
      await this.initEth()
      let balance = await window.web3.eth.getBalance(this.curAccount)
      if (10e17 * money > balance) {
        showError('The balance of account is insufficient.')
        return
      }
      const txData = {
        from: this.account,
        to: '0x42eb768f2244c8811c63729a21a3569731535f06',
        value: window.web3.utils.toWei(money.toString(), 'ether')
      }

      return new Promise((resolve, reject) => {
        window.web3.eth.sendTransaction(txData, (err, txHash) => {
          if (err) {
            reject(err)
          } else {
            // this.waitEthTransfer(txHash)
            resolve()
          }
        })
      })
    } catch (e) {
      return Promise.reject(e)
    }
  },
  // 检测转账进度
  waitEthTransfer (txHash) {
    window.web3.eth.getTransactionReceipt(txHash, (err, res) => {
      console.log('waitEthTransfer', err, res)
      if (err) {
        console.error(err)
      }

      if (!res) {
        setTimeout(this.waitEthTransfer, 1000, txHash)
      } else if (res && res.status) {
        this.visible = false
      }
      console.log('res', res)
    })
  }
}

export default ethHelper
